package path;

import java.util.ArrayList;
import java.util.Collections;
import map.Area;

public class Astar {
	
	public ArrayList<Node> open;
	public ArrayList<Node> closed;
	public Area area;
	
	public Node start;
	public Node finish;
	
	public Astar() {
		this.area = new Area(101, 101);
		this.start = new Node(20, 50, area);
		this.finish = new Node(50, 100, area);
		this.open = new ArrayList<Node>();
		this.closed = new ArrayList<Node>();
	}
	
	public void calculate() {
		
		start.distanceFromStart = 0;
		closed.clear();
		open.clear();
		open.add(start);
		
		Node current = null;
		
		while (open.size() != 0) {
			current = open.get(0);
			if (current.equals(finish)) {
				break;
			}
			open.remove(current);
			closed.add(current);
			for (Node n : current.getNeighborsList()) {
				boolean neighborIsBetter;
				if (closed.contains(n))
					continue;
				if (!n.isObstacle) {
					double neighborDistanceFromStart = current.distanceFromStart + getDistanceBetween(current, n);
					if (!open.contains(n)) {
						open.add(n);
						Collections.sort(open);
						neighborIsBetter = true;
					}
					else if (neighborDistanceFromStart < current.distanceFromStart)
						neighborIsBetter = true;
					else
						neighborIsBetter = false;
					if (neighborIsBetter) {
						n.parent = current;
						n.distanceFromStart = neighborDistanceFromStart;
						n.heuristicDistanceFromGoal = getEstimatedDistanceToGoal(n, finish);
					}
				}
				
			}
		}
		if (current != null)
			reconstructPath(current);
	}
	
	public void reconstructPath(Node node) {
		while (!(node.parent == null)) {
			System.out.printf("Node: (%d, %d)\n", node.x, node.y);
			node = node.parent;
		}
	}
	
	private double getDistanceBetween(Node n1, Node n2) {
		if ((n1.x == n2.x) || (n1.y == n2.y))
			return 1;
		else
			return 1.9;
	}
	
	private double getEstimatedDistanceToGoal(Node start, Node finish) {
		
		//Closest heuristic
		//		double dx = finish.x - start.x;
		//		double dy = finish.y - start.y;
		//		return (dx + dx) * (dy + dy);
		
		//Diagonal heuristic
		double h_diagonal = Math.min(Math.abs(start.x - finish.x), Math.abs(start.y - finish.y));
		double h_straight = Math.abs(start.x - finish.x) + Math.abs(start.y - finish.y);
		double h_result = Math.sqrt(2) * h_diagonal + (h_straight - 2 * h_diagonal);
		
		//breaking ties
		double p = 1 / 10000;
		h_result *= (1.0 + p);
		return h_result;
	}
	
	public static void main(String[] a) {
		Astar astar = new Astar();
		astar.calculate();
		System.out.println("End");
	}
}
