package path;

import java.util.ArrayList;
import map.Area;

public class Node implements Comparable<Node> {
	public boolean isClosed;
	public boolean isGoal;
	public boolean isObstacle;
	public boolean isVisited;
	public boolean isStart;
	public Node parent;
	public Node north;
	public Node south;
	public Node east;
	public Node west;
	public Node northeast;
	public Node northwest;
	public Node southeast;
	public Node southwest;
	public Area map;
	
	public int x;
	public int y;
	
	public double heuristicDistanceFromGoal;
	public double distanceFromStart;
	
	public enum Directions {
		NORTH, SOUTH, EAST, WEST, NORTHEAST, NORTHWEST, SOUTHEAST, SOUTHWEST
	};
	
	public Node(int x, int y, Area map) {
		this.x = x;
		this.y = y;
		this.isVisited = false;
		this.distanceFromStart = Integer.MAX_VALUE;
		this.isObstacle = false;
		this.isStart = false;
		this.isGoal = false;
		this.map = map;
	}
	
	public boolean equals(Node node) {
		return (this.x == node.x) && (this.y == node.y);
	}
	
	public void setNode(Directions dir, Node node) {
		Node temp = getDirectionalNode(dir);
		ArrayList<Node> list = this.getNeighborsList();
		if (list.contains(temp))
			list.remove(temp);
		list.add(node);
		setDirectionalNode(dir, node);
	}
	
	public Node getDirectionalNode(Directions dir) {
		switch (dir) {
			case NORTH:
				return north;
			case SOUTH:
				return south;
			case EAST:
				return east;
			case WEST:
				return west;
			case NORTHEAST:
				return northeast;
			case SOUTHEAST:
				return southeast;
			case NORTHWEST:
				return northwest;
			case SOUTHWEST:
				return southwest;
		}
		return null;
	}
	
	public void setDirectionalNode(Directions dir, Node node) {
		switch (dir) {
			case NORTH:
				this.north = node;
			case SOUTH:
				this.south = node;
			case EAST:
				this.east = node;
			case WEST:
				this.west = node;
			case NORTHEAST:
				this.northeast = node;
			case SOUTHEAST:
				this.southeast = node;
			case NORTHWEST:
				this.northwest = node;
			case SOUTHWEST:
				this.southwest = node;
		}
	}
	
	public ArrayList<Node> getNeighborsList() {
		ArrayList<Node> neighborList = new ArrayList<Node>();
		if (!(y == 0)) {
			neighborList.add(map.getNode(x, (y - 1)));
		}
		if (!(y == 0) && !(x == (map.width - 1))) {
			neighborList.add(map.getNode(x + 1, y - 1));
		}
		if (!(x == (map.width - 1))) {
			neighborList.add(map.getNode(x + 1, y));
		}
		if (!(x == (map.width - 1)) && !(y == (map.height - 1))) {
			neighborList.add(map.getNode(x + 1, y + 1));
		}
		if (!(y == (map.height - 1))) {
			neighborList.add(map.getNode(x, y + 1));
		}
		if (!(x == 0) && !(y == (map.height - 1))) {
			neighborList.add(map.getNode(x - 1, y + 1));
		}
		if (!(x == 0)) {
			neighborList.add(map.getNode(x - 1, y));
		}
		if (!(x == 0) && !(y == 0)) {
			neighborList.add(map.getNode(x - 1, y - 1));
		}
		return neighborList;
	}
	
	@Override
	public int compareTo(Node other) {
		double totalDistanceFromGoal = this.distanceFromStart + this.heuristicDistanceFromGoal;
		double otherDistanceFromGoal = other.distanceFromStart + other.heuristicDistanceFromGoal;
		if (totalDistanceFromGoal < otherDistanceFromGoal)
			return -1;
		if (otherDistanceFromGoal < totalDistanceFromGoal)
			return 1;
		return 0;
	}
}
